@@ -15,22 +15,17 @@ from .types import (
 
 __all__ = ("ASGIMiddleware",)
 
-global_loop = asyncio.new_event_loop()
-threading.Thread(
-    target=global_loop.run_forever, daemon=True, name="global_loop"
-).start()
-
 
 def build_scope(environ: Environ) -> Scope:
     scope = {
         "type": "http",
-        "asgi": {"version": "2.1", "spec_version": "2.1"},
+        "asgi": {"version": "3.0", "spec_version": "3.0"},
         "http_version": environ.get("SERVER_PROTOCOL", "http/1.0").split("/")[1],
         "method": environ["REQUEST_METHOD"],
         "scheme": environ.get("wsgi.url_scheme", "http"),
         "path": environ["PATH_INFO"].encode("latin1").decode("utf8"),
         "query_string": environ["QUERY_STRING"].encode("ascii"),
-        "root_path": environ.get("SCRIPT_NAME", ""),
+        "root_path": environ.get("SCRIPT_NAME", "").encode("latin1").decode("utf8"),
         "client": None,
         "server": (environ["SERVER_NAME"], int(environ["SERVER_PORT"])),
     }
@@ -62,10 +57,17 @@ class ASGIMiddleware:
         self,
         app: ASGIApp,
         wait_time: float = None,
-        loop: asyncio.AbstractEventLoop = global_loop,
+        loop: asyncio.AbstractEventLoop = None,
     ) -> None:
         self.app = app
-        self.loop = loop
+        if loop is None:
+            self.loop = asyncio.new_event_loop()
+            loop_threading = threading.Thread(
+                target=self.loop.run_forever, daemon=True, name="asgi_loop"
+            )
+            loop_threading.start()
+        else:
+            self.loop = loop
         self.wait_time = wait_time
 
     def __call__(
